@@ -52,6 +52,7 @@ class HudiEngine:
     HUDI_QUERY_TYPE_OPT_KEY = "hoodie.datasource.query.type"
     HUDI_QUERY_TYPE_SNAPSHOT_OPT_VAL = "snapshot"
     HUDI_QUERY_TYPE_INCREMENTAL_OPT_VAL = "incremental"
+    HUDI_QUERY_TYPE_SNAPSHOT_OPT_VAL = "snapshot"
     HUDI_BEGIN_INSTANTTIME_OPT_KEY = "hoodie.datasource.read.begin.instanttime"
     HUDI_END_INSTANTTIME_OPT_KEY = "hoodie.datasource.read.end.instanttime"
     PAYLOAD_CLASS_OPT_KEY = "hoodie.datasource.write.payload.class"
@@ -71,30 +72,6 @@ class HudiEngine:
         self._spark_session = spark_session
         self._feature_store_id = feature_store_id
         self._feature_store_name = feature_store_name
-        self._base_path = self._feature_group.location
-        self._table_name = feature_group._get_online_table_name()
-
-        self._primary_key = ",".join(feature_group.primary_key)
-
-        # add event time to primary key for upserts
-        if feature_group.event_time is not None:
-            self._primary_key = self._primary_key + "," + feature_group.event_time
-
-        self._partition_key = (
-            ",".join(feature_group.partition_key)
-            if len(feature_group.partition_key) >= 1
-            else ""
-        )
-        self._partition_path = (
-            ":SIMPLE,".join(feature_group.partition_key) + ":SIMPLE"
-            if len(feature_group.partition_key) >= 1
-            else ""
-        )
-        self._pre_combine_key = (
-            feature_group.hudi_precombine_key
-            if feature_group.hudi_precombine_key
-            else feature_group.primary_key[0]
-        )
 
         self._feature_group_api = feature_group_api.FeatureGroupApi(feature_store_id)
         self._storage_connector_api = storage_connector_api.StorageConnectorApi(
@@ -121,67 +98,96 @@ class HudiEngine:
         )
         return self._feature_group_api.commit(self._feature_group, fg_commit)
 
-    def register_temporary_table(
-        self, alias, start_timestamp, end_timestamp, read_options
-    ):
-        hudi_options = self._setup_hudi_read_opts(
-            start_timestamp, end_timestamp, read_options
-        )
+    def register_temporary_table(self, hudi_fg_alias, read_options):
+        hudi_options = self._setup_hudi_read_opts(hudi_fg_alias, read_options)
         self._spark_session.read.format(self.HUDI_SPARK_FORMAT).options(
             **hudi_options
-        ).load(self._base_path).createOrReplaceTempView(alias)
+        ).load("{}/**".format(self._feature_group.location)).createOrReplaceTempView(
+            hudi_fg_alias.alias
+        )
 
     def _write_hudi_dataset(self, dataset, save_mode, operation, write_options):
         hudi_options = self._setup_hudi_write_opts(operation, write_options)
         dataset.write.format(HudiEngine.HUDI_SPARK_FORMAT).options(**hudi_options).mode(
             save_mode
-        ).save(self._base_path)
+        ).save(self._feature_group.location)
 
         feature_group_commit = self._get_last_commit_metadata(
-            self._spark_context, self._base_path
+            self._spark_context, self._feature_group.location
         )
 
         return feature_group_commit
 
     def _setup_hudi_write_opts(self, operation, write_options):
-        _jdbc_url = self._get_conn_str()
+        table_name = self._feature_group._get_online_table_name()
+
+        primary_key = ",".join(self._feature_group.primary_key)
+
+        # add event time to primary key for upserts
+        if self._feature_group.event_time is not None:
+            primary_key = primary_key + "," + self._feature_group.event_time
+
+        partition_key = (
+            ",".join(self._feature_group.partition_key)
+            if len(self._feature_group.partition_key) >= 1
+            else ""
+        )
+        partition_path = (
+            ":SIMPLE,".join(self._feature_group.partition_key) + ":SIMPLE"
+            if len(self._feature_group.partition_key) >= 1
+            else ""
+        )
+        pre_combine_key = (
+            self._feature_group.hudi_precombine_key
+            if self._feature_group.hudi_precombine_key
+            else self._feature_group.primary_key[0]
+        )
+
+        jdbc_url = self._get_conn_str()
         hudi_options = {
             self.HUDI_KEY_GENERATOR_OPT_KEY: self.HUDI_COMPLEX_KEY_GENERATOR_OPT_VAL,
             self.HUDI_PRECOMBINE_FIELD: self._pre_combine_key[0]
-            if isinstance(self._pre_combine_key, list)
-            else self._pre_combine_key,
-            self.HUDI_RECORD_KEY: self._primary_key,
-            self.HUDI_PARTITION_FIELD: self._partition_path,
-            self.HUDI_TABLE_NAME: self._table_name,
+            if isinstance(pre_combine_key, list)
+            else pre_combine_key,
+            self.HUDI_RECORD_KEY: primary_key,
+            self.HUDI_PARTITION_FIELD: partition_path,
+            self.HUDI_TABLE_NAME: table_name,
             self.HIVE_PARTITION_EXTRACTOR_CLASS_OPT_KEY: self.DEFAULT_HIVE_PARTITION_EXTRACTOR_CLASS_OPT_VAL
-            if len(self._partition_key) >= 1
+            if len(partition_key) >= 1
             else self.HIVE_NON_PARTITION_EXTRACTOR_CLASS_OPT_VAL,
             self.HUDI_HIVE_SYNC_ENABLE: "true",
-            self.HUDI_HIVE_SYNC_TABLE: self._table_name,
-            self.HUDI_HIVE_SYNC_JDBC_URL: _jdbc_url,
+            self.HUDI_HIVE_SYNC_TABLE: table_name,
+            self.HUDI_HIVE_SYNC_JDBC_URL: jdbc_url,
             self.HUDI_HIVE_SYNC_DB: self._feature_store_name,
-            self.HUDI_HIVE_SYNC_PARTITION_FIELDS: self._partition_key,
+            self.HUDI_HIVE_SYNC_PARTITION_FIELDS: partition_key,
             self.HUDI_TABLE_OPERATION: operation,
             self.HUDI_HIVE_SYNC_SUPPORT_TIMESTAMP: "true",
         }
-
-        if operation.lower() in [self.HUDI_BULK_INSERT, self.HUDI_INSERT]:
-            hudi_options[self.HUDI_WRITE_INSERT_DROP_DUPLICATES] = "true"
+        hudi_options.update(HudiEngine.HUDI_DEFAULT_PARALLELISM)
 
         if write_options:
             hudi_options.update(write_options)
 
         return hudi_options
 
-    def _setup_hudi_read_opts(self, start_timestamp, end_timestamp, read_options):
-        _hudi_commit_start_time = util.get_hudi_datestr_from_timestamp(start_timestamp)
-        _hudi_commit_end_time = util.get_hudi_datestr_from_timestamp(end_timestamp)
+    def _setup_hudi_read_opts(self, hudi_fg_alias, read_options):
+        if hudi_fg_alias.left_feature_group_end_timestamp:
+            _hudi_commit_start_time = util.get_hudi_datestr_from_timestamp(
+                hudi_fg_alias.left_feature_group_start_timestamp
+            )
+            _hudi_commit_end_time = util.get_hudi_datestr_from_timestamp(
+                hudi_fg_alias.left_feature_group_end_timestamp
+            )
 
-        hudi_options = {
-            self.HUDI_QUERY_TYPE_OPT_KEY: self.HUDI_QUERY_TYPE_INCREMENTAL_OPT_VAL,
-            self.HUDI_BEGIN_INSTANTTIME_OPT_KEY: _hudi_commit_start_time,
-            self.HUDI_END_INSTANTTIME_OPT_KEY: _hudi_commit_end_time,
-        }
+            hudi_options = {
+                self.HUDI_QUERY_TYPE_OPT_KEY: self.HUDI_QUERY_TYPE_INCREMENTAL_OPT_VAL,
+                self.HUDI_BEGIN_INSTANTTIME_OPT_KEY: _hudi_commit_start_time,
+                self.HUDI_END_INSTANTTIME_OPT_KEY: _hudi_commit_end_time,
+            }
+        else:
+            hudi_options = {
+                self.HUDI_QUERY_TYPE_OPT_KEY: self.HUDI_QUERY_TYPE_SNAPSHOT_OPT_VAL,
+            }
 
         if read_options:
             hudi_options.update(read_options)
