@@ -41,12 +41,13 @@ class StatisticsApi:
             metadata_instance.id,
             "statistics",
         ]
+
         headers = {"content-type": "application/json"}
         _client._send_request(
             "POST", path_params, headers=headers, data=statistics.json()
         )
 
-    def get(self, metadata_instance, commit_timestamp):
+    def get(self, metadata_instance, commit_timestamp, for_transformation):
         """Gets the statistics for a specific commit time for an instance."""
         _client = client.get_instance()
         path_params = [
@@ -62,12 +63,14 @@ class StatisticsApi:
         query_params = {
             "filter_by": "commit_time_eq:" + str(commit_timestamp),
             "fields": "content",
+            "for_transformation": for_transformation,
         }
+
         return statistics.Statistics.from_response_json(
             _client._send_request("GET", path_params, query_params, headers=headers)
         )
 
-    def get_last(self, metadata_instance):
+    def get_last(self, metadata_instance, for_transformation):
         """Gets the statistics of the last commit for an instance."""
         _client = client.get_instance()
         path_params = [
@@ -85,6 +88,7 @@ class StatisticsApi:
             "offset": 0,
             "limit": 1,
             "fields": "content",
+            "for_transformation": for_transformation,
         }
 
         return statistics.Statistics.from_response_json(
