@@ -14,6 +14,7 @@
 #   limitations under the License.
 #
 
+import copy
 import humps
 import json
 import warnings
@@ -28,7 +29,6 @@ from hsfs.core import (
     statistics_engine,
     code_engine,
     data_validation_engine,
-    feature_group_base_engine,
     on_demand_feature_group_engine,
     expectations_api,
 )
@@ -38,11 +38,9 @@ from hsfs.client.exceptions import FeatureStoreException
 
 
 class FeatureGroupBase:
-    def __init__(self, featurestore_id, validation_type):
+    def __init__(self, featurestore_id, validation_type, location):
         self._validation_type = validation_type.upper()
-        self._feature_group_base_engine = (
-            feature_group_base_engine.FeatureGroupBaseEngine(featurestore_id)
-        )
+        self._location = location
         self._statistics_engine = statistics_engine.StatisticsEngine(
             featurestore_id, self.ENTITY_TYPE
         )
@@ -112,7 +110,8 @@ class FeatureGroupBase:
 
         # Arguments
             features: list, optional. A list of `Feature` objects or feature names as
-                strings to be selected, defaults to [], selecting all features.
+                strings to be excluded from the selection. Defaults to [],
+                selecting all features.
 
         # Returns
             `Query`: A query object with the selected features of the feature group.
@@ -177,7 +176,7 @@ class FeatureGroupBase:
             `RestAPIError` in case the backend fails to add the tag.
         """
 
-        self._feature_group_base_engine.add_tag(self, name, value)
+        self._feature_group_engine.add_tag(self, name, value)
 
     def delete_tag(self, name: str):
         """Delete a tag attached to a feature group.
@@ -188,7 +187,7 @@ class FeatureGroupBase:
         # Raises
             `RestAPIError` in case the backend fails to delete the tag.
         """
-        self._feature_group_base_engine.delete_tag(self, name)
+        self._feature_group_engine.delete_tag(self, name)
 
     def get_tag(self, name: str):
         """Get the tags of a feature group.
@@ -202,7 +201,7 @@ class FeatureGroupBase:
         # Raises
             `RestAPIError` in case the backend fails to retrieve the tag.
         """
-        return self._feature_group_base_engine.get_tag(self, name)
+        return self._feature_group_engine.get_tag(self, name)
 
     def get_tags(self):
         """Retrieves all tags attached to a feature group.
@@ -213,7 +212,7 @@ class FeatureGroupBase:
         # Raises
             `RestAPIError` in case the backend fails to retrieve the tags.
         """
-        return self._feature_group_base_engine.get_tags(self)
+        return self._feature_group_engine.get_tags(self)
 
     def get_feature(self, name: str):
         """Retrieve a `Feature` object from the schema of the feature group.
@@ -263,8 +262,12 @@ class FeatureGroupBase:
     def update_description(self, description: str):
         """Update the description of the feature group.
 
+        !!! info "Safe update"
+            This method updates the feature group description safely. In case of failure
+            your local metadata object will keep the old description.
+
         # Arguments
-            description: str. New description string.
+            description: New description string.
 
         # Returns
             `FeatureGroup`. The updated feature group object.
@@ -272,6 +275,104 @@ class FeatureGroupBase:
         self._feature_group_engine.update_description(self, description)
         return self
 
+    def update_features(self, features: Union[feature.Feature, List[feature.Feature]]):
+        """Update a single feature in this feature group.
+
+        Currently it's only supported to update the description of a feature.
+
+        !!! danger "Unsafe update"
+            Note that if you use an existing `Feature` object of the schema in the
+            feature group metadata object, this might leave your metadata object in a
+            corrupted state if the update fails.
+
+        # Arguments
+            features: `Feature` or list of features. A feature object or list thereof to
+                be updated.
+
+        # Returns
+            `FeatureGroup`. The updated feature group object.
+        """
+        new_features = []
+        if isinstance(features, feature.Feature):
+            new_features.append(features)
+        elif isinstance(features, list):
+            for feat in features:
+                if isinstance(feat, feature.Feature):
+                    new_features.append(feat)
+                else:
+                    raise TypeError(
+                        "The argument `features` has to be of type `Feature` or "
+                        "a list thereof, but an element is of type: `{}`".format(
+                            type(features)
+                        )
+                    )
+        else:
+            raise TypeError(
+                "The argument `features` has to be of type `Feature` or a list "
+                "thereof, but is of type: `{}`".format(type(features))
+            )
+        self._feature_group_engine.update_features(self, new_features)
+        return self
+
+    def update_feature_description(self, feature_name: str, description: str):
+        """Update the description of a single feature in this feature group.
+
+        !!! info "Safe update"
+            This method updates the feature description safely. In case of failure
+            your local metadata object will keep the old description.
+
+        # Arguments
+            feature_name: Name of the feature to be updated.
+            description: New description string.
+
+        # Returns
+            `FeatureGroup`. The updated feature group object.
+        """
+        f_copy = copy.deepcopy(self[feature_name])
+        f_copy.description = description
+        self._feature_group_engine.update_features(self, [f_copy])
+        return self
+
+    def append_features(self, features: Union[feature.Feature, List[feature.Feature]]):
+        """Append features to the schema of the feature group.
+
+        !!! info "Safe append"
+            This method appends the features to the feature group description safely.
+            In case of failure your local metadata object will contain the correct
+            schema.
+
+        It is only possible to append features to a feature group. Removing
+        features is considered a breaking change.
+
+        # Arguments
+            features: Feature or list. A feature object or list thereof to append to
+                the schema of the feature group.
+
+        # Returns
+            `FeatureGroup`. The updated feature group object.
+        """
+        new_features = []
+        if isinstance(features, feature.Feature):
+            new_features.append(features)
+        elif isinstance(features, list):
+            for feat in features:
+                if isinstance(feat, feature.Feature):
+                    new_features.append(feat)
+                else:
+                    raise TypeError(
+                        "The argument `features` has to be of type `Feature` or "
+                        "a list thereof, but an element is of type: `{}`".format(
+                            type(features)
+                        )
+                    )
+        else:
+            raise TypeError(
+                "The argument `features` has to be of type `Feature` or a list "
+                "thereof, but is of type: `{}`".format(type(features))
+            )
+        self._feature_group_engine.append_features(self, new_features)
+        return self
+
     def attach_expectation(self, expectation):
         """Attach a feature group expectation. If feature group validation is not already enabled, it will be enabled
         and set to the stricter setting.
@@ -285,7 +386,7 @@ class FeatureGroupBase:
         """
         # Turn on validation for this FG and set stricter setting
         if self._validation_type == "NONE":
-            self._validation_type = "STRICT"
+            self.validation_type = "STRICT"
 
         return self._expectations_api.attach(self, expectation.name)
 
@@ -451,6 +552,10 @@ class FeatureGroupBase:
     def event_time(self, feature_name):
         self._event_time = feature_name
 
+    @property
+    def location(self):
+        return self._location
+
 
 class FeatureGroup(FeatureGroupBase):
     CACHED_FEATURE_GROUP = "CACHED_FEATURE_GROUP"
@@ -479,7 +584,7 @@ class FeatureGroup(FeatureGroupBase):
         online_topic_name=None,
         event_time=None,
     ):
-        super().__init__(featurestore_id, validation_type)
+        super().__init__(featurestore_id, validation_type, location)
 
         self._feature_store_id = featurestore_id
         self._feature_store_name = featurestore_name
@@ -494,7 +599,6 @@ class FeatureGroup(FeatureGroupBase):
             for feat in (features or [])
         ]
 
-        self._location = location
         self._online_enabled = online_enabled
         self._time_travel_format = (
             time_travel_format.upper() if time_travel_format is not None else None
@@ -717,9 +821,6 @@ class FeatureGroup(FeatureGroupBase):
                 * key `wait_for_job` and value `True` or `False` to configure
                   whether or not to the save call should return only
                   after the Hopsworks Job has finished. By default it waits.
-                * key `mode` instruct the ingestion job on how to deal with corrupted
-                  data. Values are PERMISSIVE, DROPMALFORMED or FAILFAST. Default FAILFAST.
-
 
         # Returns
             `Job`: When using the `hive` engine, it returns the Hopsworks Job
@@ -806,8 +907,6 @@ class FeatureGroup(FeatureGroupBase):
                 * key `wait_for_job` and value `True` or `False` to configure
                   whether or not to the insert call should return only
                   after the Hopsworks Job has finished. By default it waits.
-                * key `mode` instruct the ingestion job on how to deal with corrupted
-                  data. Values are PERMISSIVE, DROPMALFORMED or FAILFAST. Default FAILFAST.
 
         # Returns
             `FeatureGroup`. Updated feature group metadata object.
@@ -836,6 +935,7 @@ class FeatureGroup(FeatureGroupBase):
         output_mode: Optional[str] = "append",
         await_termination: Optional[bool] = False,
         timeout: Optional[int] = None,
+        checkpoint_dir: Optional[str] = None,
         write_options: Optional[Dict[Any, Any]] = {},
     ):
         """Ingest a Spark Structured Streaming Dataframe to the online feature store.
@@ -871,9 +971,12 @@ class FeatureGroup(FeatureGroupBase):
                 query.stop() or by an exception. If the query has terminated with an
                 exception, then the exception will be thrown. If timeout is set, it
                 returns whether the query has terminated or not within the timeout
-                seconds. Defaults to `Fals`e.
+                seconds. Defaults to `False`.
             timeout: Only relevant in combination with `await_termination=True`.
                 Defaults to `None`.
+            checkpoint_dir: Checkpoint directory location. This will be used to as a reference to
+            from where to resume the streaming job. If `None` then hsfs will construct as
+            "insert_stream_" + online_topic_name. Defaults to `None`.
             write_options: Additional write options for Spark as key-value pairs.
                 Defaults to `{}`.
 
@@ -906,6 +1009,7 @@ class FeatureGroup(FeatureGroupBase):
                 output_mode,
                 await_termination,
                 timeout,
+                checkpoint_dir,
                 write_options,
             )
 
@@ -969,68 +1073,28 @@ class FeatureGroup(FeatureGroupBase):
         """
         return self.select_all().as_of(wallclock_time)
 
-    def update_description(self, description: str):
-        """Update the description of the feature gorup.
-
-        # Arguments
-            description: str. New description string.
-
-        # Returns
-            `FeatureGroup`. The updated feature group object.
-        """
-        self._feature_group_engine.update_description(self, description)
-        return self
-
-    def append_features(self, features):
-        """Append features to the schema of the feature group.
-
-        It is only possible to append features to a feature group. Removing
-        features is considered a breaking change.
-
-        # Arguments
-            features: Feature or list. A feature object or list thereof to append to
-                the schema of the feature group.
-
-        # Returns
-            FeatureGroup. The updated feature group object.
-        """
-        new_features = []
-        if isinstance(features, feature.Feature):
-            new_features.append(features)
-        elif isinstance(features, list):
-            for feat in features:
-                if isinstance(feat, feature.Feature):
-                    new_features.append(feat)
-                else:
-                    raise TypeError(
-                        "The argument `features` has to be of type `Feature` or "
-                        "a list thereof, but an element is of type: `{}`".format(
-                            type(features)
-                        )
-                    )
-        else:
-            raise TypeError(
-                "The argument `features` has to be of type `Feature` or a list "
-                "thereof, but is of type: `{}`".format(type(features))
-            )
-        self._feature_group_engine.append_features(self, new_features)
-        return self
-
     def validate(
-        self, dataframe: TypeVar("pyspark.sql.DataFrame") = None  # noqa: F821
+        self,
+        dataframe: TypeVar("pyspark.sql.DataFrame") = None,  # noqa: F821
+        log_activity=False,  # noqa: F821
     ):
         """Run validation based on the attached expectations
 
         # Arguments
             dataframe: The PySpark dataframe to run the data validation expectations against.
+            log_activity: Whether to log the validation as a feature group activity.
+                If a dataframe is not provided (None),
+                the validation will be logged as a feature store activity.
 
         # Returns
             `FeatureGroupValidation`. The feature group validation metadata object.
 
         """
+        # Activity is logged only if a the validation concerts the feature group and not a specific dataframe
         if dataframe is None:
             dataframe = self.read()
-        return self._data_validation_engine.validate(self, dataframe)
+            log_activity = True
+        return self._data_validation_engine.validate(self, dataframe, log_activity)
 
     def compute_statistics(self, wallclock_time: Optional[str] = None):
         """Recompute the statistics for the feature group and save them to the
@@ -1175,10 +1239,6 @@ class FeatureGroup(FeatureGroupBase):
         """Schema information."""
         return self._features
 
-    @property
-    def location(self):
-        return self._location
-
     @property
     def online_enabled(self):
         """Setting if the feature group is available in online storage."""
@@ -1298,12 +1358,13 @@ class OnDemandFeatureGroup(FeatureGroupBase):
         creator=None,
         id=None,
         features=None,
+        location=None,
         statistics_config=None,
         event_time=None,
         validation_type="NONE",
         expectations=None,
     ):
-        super().__init__(featurestore_id, validation_type)
+        super().__init__(featurestore_id, validation_type, location)
 
         self._feature_store_id = featurestore_id
         self._feature_store_name = featurestore_name
@@ -1318,6 +1379,11 @@ class OnDemandFeatureGroup(FeatureGroupBase):
         self._id = id
         self._event_time = event_time
 
+        self._features = [
+            feature.Feature.from_response_json(feat) if isinstance(feat, dict) else feat
+            for feat in (features or [])
+        ]
+
         self._feature_group_engine = (
             on_demand_feature_group_engine.OnDemandFeatureGroupEngine(featurestore_id)
         )
@@ -1397,7 +1463,7 @@ class OnDemandFeatureGroup(FeatureGroupBase):
             `FeatureGroupValidation`. The feature group validation metadata object.
 
         """
-        return self._data_validation_engine.validate(self, self.read())
+        return self._data_validation_engine.validate(self, self.read(), True)
 
     @classmethod
     def from_response_json(cls, json_dict):
